@@ -21,10 +21,6 @@ class Database:
         self.type_features = type_features
         self.type_classes = type_classes
 
-        self.table_params = table_params
-        self.table_corpus = table_corpus
-        self.table_weights = table_weights
-
         self.j = self.field_features = field_features
         self.k = self.field_classes = field_classes
         self.n = self.field_items = field_items
@@ -36,7 +32,7 @@ class Database:
             self.engine = engine
 
         self.table_params = Table(
-            f"{self.prefix}_{self.table_params}", MetaData(),
+            f"{self.prefix}_{table_params}", MetaData(),
             Column('prefix', String, primary_key=True),
             Column('a', Float, nullable=False),
             Column('b', Float, nullable=False),
@@ -44,29 +40,24 @@ class Database:
         )
 
         self.table_corpus = Table(
-            f"{self.prefix}_{self.table_corpus}", MetaData(),
+            f"{self.prefix}_{table_corpus}", MetaData(),
             Column(self.field_features, self.type_features, primary_key=True),
             Column(self.field_classes, self.type_classes, primary_key=True),
             Column(self.field_weights, Float, nullable=False),
         )
 
         self.table_weights = Table(
-            f"{self.prefix}_{self.table_weights}", MetaData(),
+            f"{self.prefix}_{table_weights}", MetaData(),
             Column(self.field_features, self.type_features, primary_key=True),
             Column(self.field_classes, self.type_classes, primary_key=True),
             Column(self.field_weights, Float, nullable=False),
         )
 
-        if not self.exists(self.table_params):
-            with self.connect() as con:
-                with con.begin():
-                    self.write_params(con, a=0.5, b=1, h=1)
-
     def connect(self):
         return self.engine.connect()
 
-    def exists(self, table):
-        return inspect(self.engine).has_table(table.name)
+    def exists(self, con, table):
+        return inspect(con).has_table(table.name)
 
     def write(self, con, table, values=None, if_exists='fail', **kwargs):
         assert if_exists in {
@@ -78,7 +69,7 @@ class Database:
             'insert_or_sum'
         }
 
-        if not self.exists(table):
+        if not self.exists(con, table):
             table.create(con)
 
         else:
@@ -221,28 +212,34 @@ class Database:
 
         return self.write(con, table=table, values=values)
 
-    def is_deployed(self):
-        return self.exists(self.table_weights)
+    def is_params(self, con):
+        return self.exists(con, self.table_params)
 
-    def is_fitted(self):
-        return self.exists(self.table_corpus) or self.is_deployed()
+    def is_corpus(self, con):
+        return self.exists(con, self.table_corpus)
 
-    def check_fitted(self):
-        if not self.is_fitted():
+    def is_deployed(self, con):
+        return self.exists(con, self.table_weights)
+
+    def is_fitted(self, con):
+        return self.is_corpus(con) or self.is_deployed(con)
+
+    def check_fitted(self, con):
+        if not self.is_fitted(con):
             raise ValueError(
                 f"This instance is not fitted yet."
             )
 
-    def check_editable(self):
-        if self.is_deployed():
+    def check_editable(self, con):
+        if self.is_deployed(con):
             raise ValueError(
                 "Cannot modify a deployed instance."
             )
 
-    def deploy(self, con):
-        self.check_fitted()
+    def deploy(self, con, deep):
+        self.check_fitted(con)
 
-        if self.is_deployed():
+        if self.is_deployed(con):
             raise ValueError(
                 "This instance is already deployed. Nothing to do."
             )
@@ -257,35 +254,43 @@ class Database:
         self.write(con, table=self.table_weights)
         con.execute(text(sql))
 
-    def undeploy(self, con):
-        if not self.exists(self.table_corpus):
+        if deep:
+            self.table_corpus.drop(con)
+
+    def undeploy(self, con, deep):
+        if not deep and not self.exists(con, self.table_corpus):
             raise ValueError(
-                "This instance has no corpus. Cannot undeploy: information would be lost."
+                "This instance has no corpus and the model would be lost. "
+                "Set deep=True to force undeploy."
             )
 
-        if not self.is_deployed():
+        if not deep and not self.is_deployed(con):
             raise ValueError(
                 "This instance is already undeployed. Nothing to do."
             )
 
-        self.table_weights.drop(con)
+        self.table_weights.drop(con, checkfirst=True)
+
+        if deep:
+            self.table_params.drop(con, checkfirst=True)
+            self.table_corpus.drop(con, checkfirst=True)
 
     def predict(self, con, X):
-        cache = self.is_deployed()
+        cache = self.is_deployed(con)
         items = self.write_items(con, X)
         sql = self._sql_predict(cache, items)
 
         return self.read_sql(sql, con)
 
     def predict_proba(self, con, X):
-        cache = self.is_deployed()
+        cache = self.is_deployed(con)
         items = self.write_items(con, X)
         sql = self._sql_predict_proba(cache, items)
 
         return self.read_sql(sql, con)
 
     def explain(self, con, X=None, sample_weight=None):
-        cache = self.is_deployed()
+        cache = self.is_deployed(con)
 
         if X is None:
             sql = self._sql_explain(cache)
